import os
import sys
import time
import schedule
from WeatherTracker import ROOT_PATH
from WeatherTracker.weather.get_weather_data import GetWeatherData
from WeatherTracker.weather.write_weather_data import WriteWeatherData


class TrackWeather:
    def __init__(self, lat, lon, interval):
        print('Running...')
        self.lat = lat
        self.lon = lon
        self.interval = interval
        self.data = None
        self.last_dt = None
        self.run()

    def run(self):
        self.data = GetWeatherData(self.lat, self.lon)
        schedule.every(self.interval).minutes.do(self.job)
        while True:
            schedule.run_pending()
            time.sleep(1)

    def job(self):
        try:
            self.data.now()
            print()
            self.data.print_data()
            if self.last_dt is None or self.last_dt != self.data.dt:
                print('writing new data')
                WriteWeatherData(self.data)
            else:
                print('repeat data detected')
            self.last_dt = self.data.dt
        except Exception as e:  # If an exception occurs during model application
            print(f"Exception when retrieving data: {e}")  # Print exception
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print("Exception type:", exc_type, "\nFile:", f"{fname},", "Line", exc_tb.tb_lineno)


if __name__ == '__main__':
    # # Urbana
    # job = TrackWeather(lat=40.1106, lon=-88.2073, interval=3)

    # # Pekin
    # job = TrackWeather(lat=40.5675, lon=-89.6407, interval=3)

    # Denver
    # job = TrackWeather(lat=39.7392, lon=-104.9903, interval=3)

    # # Memphis TN
    # job = TrackWeather(lat=35.1495, lon=-90.0490, interval=3)

    # Ogrlthorpe
    job = TrackWeather(lat=40.103180704025185, lon=-88.17479134286376, interval=3)



import numpy as np


def movmean(arr, N: int = 3):
    """
    Compute centered moving average of 1d array. Averaging window is asymmetric for elements N/2 from the beginning and end.
    :param arr: numpy array to be analyzed
    :type arr: numpy.ndarray
    :param N: total width of centered moving average window
    :type N: int
    :return: numpy.ndarray containing moving average of arr
    """
    ret = np.array([])
    for i in range(len(arr)):
        if N % 2 == 0:
            n = int(N / 2)
            if i - n < 0:
                val = np.sum(arr[0:i + n]) / len(arr[0:i + n])
            elif i + n > len(arr) - 1:
                val = np.sum(arr[i - n:len(arr)]) / len(arr[i - n:len(arr)])
            else:
                val = np.sum(arr[i - n:i + n]) / N
        else:
            n = int((N - 1) / 2)
            if i - n < 0:
                val = np.sum(arr[0:i + n + 1]) / len(arr[0:i + n + 1])
            elif i + n > len(arr) - 1:
                val = np.sum(arr[i - n:len(arr)]) / len(arr[i - n:len(arr)])
            else:
                val = np.sum(arr[i - n:i + n + 1]) / N
        ret = np.append(ret, val)
    return ret


import os
import csv
import shutil
from datetime import datetime
from WeatherTracker import ROOT_PATH


class WriteWeatherData:
    def __init__(self, data):
        self.data = data
        self.dir_path = None
        self.write()

    def write(self):
        dt = datetime.now()
        date_str = f'{str(dt.year)}{str(dt.month).zfill(2)}{str(dt.day).zfill(2)}'
        self.dir_path = os.path.join(ROOT_PATH, 'data', f'{self.data.city}')
        self.make_dir()  # create new directory if necessary
        data_path = os.path.join(self.dir_path, f'{date_str}_{self.data.city}.csv')
        row = [self.data.dt, self.data.lat, self.data.lon, self.data.t_now, self.data.t_feel, self.data.humid,
               self.data.p_now, self.data.wind_vel, self.data.wind_dir, self.data.clouds, self.data.params,
               self.data.descript, str(dt)]
        if not os.path.exists(data_path):
            with open(data_path, 'w') as f:
                col_labels = ['datetime', 'latitude', 'longitude', 'temperature (K)', 'feels like temperature (K)',
                              'humidity (%)', 'pressure (kPa)', 'wind speed (m/s)', 'wind direction (deg)',
                              'cloudiness (%)', 'parameters', 'description', 'retrieval datetime']
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(col_labels)
                writer.writerow(row)
        else:
            with open(data_path, 'a') as f:
                writer = csv.writer(f, lineterminator='\n')
                writer.writerow(row)

    def make_dir(self):
        if not os.path.isdir(self.dir_path):  # Check if directory exists already
            os.mkdir(self.dir_path)  # Create new empty directory




import os
import sys
import json
import numpy as np
import pandas as pd
import qdarktheme
import pyqtgraph as pg
from PyQt5 import QtCore, QtWidgets
from pyqtgraph import PlotWidget, plot
from PyQt5.QtGui import QFont, QPalette
from PyQt5.QtCore import QSize, QFile, QTextStream
from PyQt5.QtWidgets import QMainWindow, QWidget, QLabel, QLineEdit, QMessageBox, QComboBox, QApplication, \
    QVBoxLayout, QPushButton, QCheckBox, QApplication
from WeatherTracker import ROOT_PATH
from WeatherTracker.functions.movmean import movmean


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.degree_sign = u'\N{DEGREE SIGN}'

        # Window
        width = 1000
        height = 1000
        pad = 100
        self.setMinimumSize(QSize(width, height))
        self.setWindowTitle("Weather Display")

        # Plot
        self.graphWidget = pg.PlotWidget()
        self.setCentralWidget(self.graphWidget)
        self.graphWidget.resize(width-2*pad, height-2*pad)
        # self.graphWidget.move(pad, pad)
        self.read_data()
        self.graphWidget.plot(np.arange(len(self.df1['datetime'])),
                              movmean((self.df1['temperature (K)']-273.15)*9/5+32, N=5), color=(255, 0, 0))
        self.graphWidget.plot(np.arange(len(self.df2['datetime'])),
                              movmean((self.df2['temperature (K)']-273.15)*9/5+32, N=5), color=(0, 255, 0))
        self.graphWidget.setLabel('left', f'Temperature ({self.degree_sign}C)')
        self.graphWidget.setLabel('bottom', 'Time Point')

    def read_data(self):
        self.df1 = pd.read_csv(os.path.join(ROOT_PATH, 'data', 'Pekin.csv'))
        self.df2 = pd.read_csv(os.path.join(ROOT_PATH, 'data', 'Urbana.csv'))


if __name__ == '__main__':
    app = QApplication(sys.argv)
    # app.setStyleSheet(qdarktheme.load_stylesheet())
    w = MainWindow()
    w.show()
    app.exec_()



import setuptools

with open("WeatherTracker/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="WeatherTracker",
    version="0.0.1",
    author="Benjamin Ringel",
    author_email="<benringel22@gmail.com>",
    description="Personal project for tracking weather",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/bringel2/WeatherTracker",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



# import required modules
import json
import requests
import schedule
import numpy as np
from datetime import datetime


class GetWeatherData:
	def __init__(self, lat, lon):
		self.api_key = "10d274e4344f8f19dab6bbceb9e5232f"
		self.lat = lat
		self.lon = lon

		self.response = None
		self.data = None
		self.dt = None
		self.t_now = None
		self.t_feel = None
		self.t_max = None
		self.t_min = None
		self.p_now = None
		self.humid = None
		self.clouds = None
		self.params = None
		self.descript = None
		self.wind_vel = None
		self.wind_dir = None
		self.wind_gust = None
		self.city = None

	def now(self):
		url = f"http://api.openweathermap.org/data/2.5/weather?" \
			f"appid={self.api_key}&lat={str(self.lat)}&lon={str(self.lon)}"
		self.response = requests.get(url)
		self.data = self.response.json()

		if self.data["cod"] != "404":  # check that city is found
			self.dt = datetime.fromtimestamp(self.data['dt'])		# Time of call converted to datetime object
			self.t_now = self.data["main"]["temp"]  	   			# Current temperature [K]
			self.t_feel = self.data["main"]["feels_like"]  			# Current feels like temperature [K]
			self.t_max = self.data["main"]["temp_max"]	   			# High temperature for the day [K]
			self.t_min = self.data["main"]["temp_min"]	   			# Low temperature for the day [K]
			self.p_now = self.data["main"]["pressure"]/10  			# Current pressure [kPa]
			self.humid = self.data["main"]["humidity"]	   			# Current humidity [%]
			self.lon = self.data["coord"]["lon"]  					# Longitude location [deg]
			self.lat = self.data["coord"]["lat"]  					# Latitude location [deg]
			self.clouds = self.data['clouds']['all']				# Cloudiness [%]
			self.params = self.data["weather"][0]["main"]  			# Qualitative description 1
			self.descript = self.data["weather"][0]["description"]  # Qualitative description 2
			self.wind_vel = self.data['wind']['speed']  			# Wind speed [m/s]
			self.wind_dir = self.data['wind']['deg']  				# Wind direction [deg]
			self.city = self.data['name']  							# City name

	# def forecast(self, cnt: int = 8):
	# 	self.cnt = cnt  # number of days to return [1 to 5]
	# 	url = f"http://api.openweathermap.org/data/2.5/forecast?" \
	# 		f"lat={str(self.lat)}&lon={str(self.lon)}&appid={self.api_key}&cnt={str(cnt)}"
	# 	self.response = requests.get(url)
	# 	self.data = self.response.json()
	#
	# 	self.t_arr = np.array([])
	# 	self.t_feel_arr = np.array([])
	# 	self.p_arr = np.array([])
	# 	self.humid_arr = np.array([])
	# 	self.main_arr = np.array([])
	#
	# 	for x in self.data['list']:
	# 		self.t_arr = np.append(self.t_arr, x['main']['temp'])
	# 		self.main_arr = np.append(self.main_arr, x['weather'][0]['main'])

	def print_data(self):
		t = self.datetime_to_str(self.dt, military=False)
		print(f'Weather in {self.city} at {t["hour"]}:{t["min"]}{t["meridian"]}, {t["month"]}/{t["day"]}/{t["year"]} (called {datetime.now()})')
		print(f'Temperature: {round(self.convert_temp(self.t_now, "f"), 3)}' + u'\N{DEGREE SIGN}F')
		print(f'Feels like:  {round(self.convert_temp(self.t_feel, "f"), 3)}' + u'\N{DEGREE SIGN}F')
		print(f'Wind speed:  {round(self.wind_vel * 2.237, 2)} mph')
		print(f'Humidity:    {self.humid}%')
		print(f'Pressure:    {self.p_now} kPa')
		print(f'Cloudiness:  {self.clouds}%')

	@staticmethod
	def convert_temp(T_in, units='C'):
		"""
		Converts temperature in K to deg F or deg C.
		:param T_in: Input temperature in K
		:type T_in: float
		:param units: Units to convert to ('C' or 'F')
		:return units: str
		"""
		if units.lower() == 'c':
			return T_in - 273.15
		elif units.lower() == 'f':
			return (T_in - 273.15)*9/5 + 32
		else:
			raise ValueError("'Unit' invalid.")

	@staticmethod
	def datetime_to_str(dt, military=True):
		hour = dt.hour
		if not military:
			if hour >= 12:
				meridian = 'pm'
				hour = hour - 12 if hour > 12 else hour
			else:
				meridian = 'am'
				hour = 12 if hour < 1 else hour
		else:
			meridian = ''

		dictionary = {
			'year': str(dt.year),
			'month': str(dt.month),
			'day': str(dt.day).zfill(2),
			'hour': str(hour) if not military else str(hour).zfill(2),
			'min': str(dt.minute).zfill(2),
			'sec': str(dt.second).zfill(2),
			'meridian': meridian,
					}
		return dictionary


if __name__ == '__main__':
	# Urbana weather
	lat = 40.103180704025185
	lon = -88.17479134286376
	data = GetWeatherData(lat=lat, lon=lon)
	data.now()
	data.print_data()
	# data.forecast()
